open Ext
open Scanf
open Printf

let is_whitespace = function
 | ' ' | '\n' | '\r' | '\t' -> true
 | _ -> false

let input_token ic buf len =
  (* Skip leading whitespace *)
  let c = ref ' ' in
  while is_whitespace !c do
    c := input_char ic
  done;
  (* Read token *)
  let i = ref 0 in
  while !i < len && not (is_whitespace !c) do
    String.set buf !i !c;
    c := input_char ic;
    incr i
  done;
  let s = String.create !i in
  String.blit buf 0 s 0 !i;
  s

let input_buf = String.create 1024

let input_int ic = 
  int_of_string (input_token ic input_buf 1024)

let input_float ic =
  float_of_string (input_token ic input_buf 1024)

let input_word ic =
  input_token ic input_buf 1024

let input ic =
  (* Read in "Markov" or "Bayes" *)
  let w = String.uppercase (input_word ic) in
  if String.compare "MARKOV" w <> 0 
    && String.compare "BAYES" w <> 0 then begin
    (* TODO: Friendlier notification of errors... *)
    print_string "ERROR: UAI file does not start with MARKOV or BAYES.\n";
    flush stdout;
    raise Parsing.Parse_error
  end;

  (* Read schema *)
  let num_vars = input_int ic in 
  let schema = Array.init num_vars (fun i -> input_int ic) in

  (* Read factor scopes *)
  let num_factors = input_int ic in
  let read_scope i =
    let num_args = input_int ic in
    Array.init num_args (fun j -> input_int ic) in
  let scopes = Array.init num_factors (fun i -> read_scope i) in

  (* Read factor values *)
  let read_factor i =
    let num_values = input_int ic in
    Array.init num_values (fun j -> log (input_float ic)) in
  let factor_values = Array.init num_factors (fun i -> read_factor i) in

  (* Build factors (tables only) *)
  let range i = schema.(i) in 
  let build_table scope values =
    Factor.Table (scope, (Array.map range scope), values) in
  let factors = Array.map2 build_table scopes factor_values in
  (schema, factors)

let output o schema factors =
  output_string o "MARKOV\n";
  fprintf o "%d\n" (Array.length schema);
  fprintf o "%s\n" (string_of_iarray schema);
  fprintf o "%d\n" (Array.length factors);
  let print_scope = function
  | Factor.Table (vars, ranges, values) ->
    fprintf o "%d %s\n" (Array.length vars) (string_of_iarray vars)
  | _ -> 
    print_string "ERROR: only table factors are supported.\n";
    flush stdout;
    assert false (* TODO: Better error reporting... *) in
  Array.iter print_scope factors;

  let print_values = function
  | Factor.Table (vars, ranges, values) ->
    fprintf o "\n%d %s\n" (Array.length values) (string_of_farray values)
  | _ -> assert false in
  Array.iter print_values factors
